# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import IsaacusError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import embeddings, rerankings, enrichments, extractions, classifications
    from .resources.embeddings import EmbeddingsResource, AsyncEmbeddingsResource
    from .resources.rerankings import RerankingsResource, AsyncRerankingsResource
    from .resources.enrichments import EnrichmentsResource, AsyncEnrichmentsResource
    from .resources.extractions.extractions import ExtractionsResource, AsyncExtractionsResource
    from .resources.classifications.classifications import ClassificationsResource, AsyncClassificationsResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Isaacus", "AsyncIsaacus", "Client", "AsyncClient"]


class Isaacus(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Isaacus client instance.

        This automatically infers the `api_key` argument from the `ISAACUS_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ISAACUS_API_KEY")
        if api_key is None:
            raise IsaacusError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ISAACUS_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ISAACUS_BASE_URL")
        if base_url is None:
            base_url = f"https://api.isaacus.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        from .resources.embeddings import EmbeddingsResource

        return EmbeddingsResource(self)

    @cached_property
    def classifications(self) -> ClassificationsResource:
        from .resources.classifications import ClassificationsResource

        return ClassificationsResource(self)

    @cached_property
    def rerankings(self) -> RerankingsResource:
        from .resources.rerankings import RerankingsResource

        return RerankingsResource(self)

    @cached_property
    def extractions(self) -> ExtractionsResource:
        from .resources.extractions import ExtractionsResource

        return ExtractionsResource(self)

    @cached_property
    def enrichments(self) -> EnrichmentsResource:
        from .resources.enrichments import EnrichmentsResource

        return EnrichmentsResource(self)

    @cached_property
    def with_raw_response(self) -> IsaacusWithRawResponse:
        return IsaacusWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IsaacusWithStreamedResponse:
        return IsaacusWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncIsaacus(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncIsaacus client instance.

        This automatically infers the `api_key` argument from the `ISAACUS_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ISAACUS_API_KEY")
        if api_key is None:
            raise IsaacusError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ISAACUS_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ISAACUS_BASE_URL")
        if base_url is None:
            base_url = f"https://api.isaacus.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        from .resources.embeddings import AsyncEmbeddingsResource

        return AsyncEmbeddingsResource(self)

    @cached_property
    def classifications(self) -> AsyncClassificationsResource:
        from .resources.classifications import AsyncClassificationsResource

        return AsyncClassificationsResource(self)

    @cached_property
    def rerankings(self) -> AsyncRerankingsResource:
        from .resources.rerankings import AsyncRerankingsResource

        return AsyncRerankingsResource(self)

    @cached_property
    def extractions(self) -> AsyncExtractionsResource:
        from .resources.extractions import AsyncExtractionsResource

        return AsyncExtractionsResource(self)

    @cached_property
    def enrichments(self) -> AsyncEnrichmentsResource:
        from .resources.enrichments import AsyncEnrichmentsResource

        return AsyncEnrichmentsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncIsaacusWithRawResponse:
        return AsyncIsaacusWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIsaacusWithStreamedResponse:
        return AsyncIsaacusWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class IsaacusWithRawResponse:
    _client: Isaacus

    def __init__(self, client: Isaacus) -> None:
        self._client = client

    @cached_property
    def embeddings(self) -> embeddings.EmbeddingsResourceWithRawResponse:
        from .resources.embeddings import EmbeddingsResourceWithRawResponse

        return EmbeddingsResourceWithRawResponse(self._client.embeddings)

    @cached_property
    def classifications(self) -> classifications.ClassificationsResourceWithRawResponse:
        from .resources.classifications import ClassificationsResourceWithRawResponse

        return ClassificationsResourceWithRawResponse(self._client.classifications)

    @cached_property
    def rerankings(self) -> rerankings.RerankingsResourceWithRawResponse:
        from .resources.rerankings import RerankingsResourceWithRawResponse

        return RerankingsResourceWithRawResponse(self._client.rerankings)

    @cached_property
    def extractions(self) -> extractions.ExtractionsResourceWithRawResponse:
        from .resources.extractions import ExtractionsResourceWithRawResponse

        return ExtractionsResourceWithRawResponse(self._client.extractions)

    @cached_property
    def enrichments(self) -> enrichments.EnrichmentsResourceWithRawResponse:
        from .resources.enrichments import EnrichmentsResourceWithRawResponse

        return EnrichmentsResourceWithRawResponse(self._client.enrichments)


class AsyncIsaacusWithRawResponse:
    _client: AsyncIsaacus

    def __init__(self, client: AsyncIsaacus) -> None:
        self._client = client

    @cached_property
    def embeddings(self) -> embeddings.AsyncEmbeddingsResourceWithRawResponse:
        from .resources.embeddings import AsyncEmbeddingsResourceWithRawResponse

        return AsyncEmbeddingsResourceWithRawResponse(self._client.embeddings)

    @cached_property
    def classifications(self) -> classifications.AsyncClassificationsResourceWithRawResponse:
        from .resources.classifications import AsyncClassificationsResourceWithRawResponse

        return AsyncClassificationsResourceWithRawResponse(self._client.classifications)

    @cached_property
    def rerankings(self) -> rerankings.AsyncRerankingsResourceWithRawResponse:
        from .resources.rerankings import AsyncRerankingsResourceWithRawResponse

        return AsyncRerankingsResourceWithRawResponse(self._client.rerankings)

    @cached_property
    def extractions(self) -> extractions.AsyncExtractionsResourceWithRawResponse:
        from .resources.extractions import AsyncExtractionsResourceWithRawResponse

        return AsyncExtractionsResourceWithRawResponse(self._client.extractions)

    @cached_property
    def enrichments(self) -> enrichments.AsyncEnrichmentsResourceWithRawResponse:
        from .resources.enrichments import AsyncEnrichmentsResourceWithRawResponse

        return AsyncEnrichmentsResourceWithRawResponse(self._client.enrichments)


class IsaacusWithStreamedResponse:
    _client: Isaacus

    def __init__(self, client: Isaacus) -> None:
        self._client = client

    @cached_property
    def embeddings(self) -> embeddings.EmbeddingsResourceWithStreamingResponse:
        from .resources.embeddings import EmbeddingsResourceWithStreamingResponse

        return EmbeddingsResourceWithStreamingResponse(self._client.embeddings)

    @cached_property
    def classifications(self) -> classifications.ClassificationsResourceWithStreamingResponse:
        from .resources.classifications import ClassificationsResourceWithStreamingResponse

        return ClassificationsResourceWithStreamingResponse(self._client.classifications)

    @cached_property
    def rerankings(self) -> rerankings.RerankingsResourceWithStreamingResponse:
        from .resources.rerankings import RerankingsResourceWithStreamingResponse

        return RerankingsResourceWithStreamingResponse(self._client.rerankings)

    @cached_property
    def extractions(self) -> extractions.ExtractionsResourceWithStreamingResponse:
        from .resources.extractions import ExtractionsResourceWithStreamingResponse

        return ExtractionsResourceWithStreamingResponse(self._client.extractions)

    @cached_property
    def enrichments(self) -> enrichments.EnrichmentsResourceWithStreamingResponse:
        from .resources.enrichments import EnrichmentsResourceWithStreamingResponse

        return EnrichmentsResourceWithStreamingResponse(self._client.enrichments)


class AsyncIsaacusWithStreamedResponse:
    _client: AsyncIsaacus

    def __init__(self, client: AsyncIsaacus) -> None:
        self._client = client

    @cached_property
    def embeddings(self) -> embeddings.AsyncEmbeddingsResourceWithStreamingResponse:
        from .resources.embeddings import AsyncEmbeddingsResourceWithStreamingResponse

        return AsyncEmbeddingsResourceWithStreamingResponse(self._client.embeddings)

    @cached_property
    def classifications(self) -> classifications.AsyncClassificationsResourceWithStreamingResponse:
        from .resources.classifications import AsyncClassificationsResourceWithStreamingResponse

        return AsyncClassificationsResourceWithStreamingResponse(self._client.classifications)

    @cached_property
    def rerankings(self) -> rerankings.AsyncRerankingsResourceWithStreamingResponse:
        from .resources.rerankings import AsyncRerankingsResourceWithStreamingResponse

        return AsyncRerankingsResourceWithStreamingResponse(self._client.rerankings)

    @cached_property
    def extractions(self) -> extractions.AsyncExtractionsResourceWithStreamingResponse:
        from .resources.extractions import AsyncExtractionsResourceWithStreamingResponse

        return AsyncExtractionsResourceWithStreamingResponse(self._client.extractions)

    @cached_property
    def enrichments(self) -> enrichments.AsyncEnrichmentsResourceWithStreamingResponse:
        from .resources.enrichments import AsyncEnrichmentsResourceWithStreamingResponse

        return AsyncEnrichmentsResourceWithStreamingResponse(self._client.enrichments)


Client = Isaacus

AsyncClient = AsyncIsaacus



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["AnswerExtractionResponse", "Extraction", "ExtractionAnswer", "Usage"]


class ExtractionAnswer(BaseModel):
    """An answer extracted from a text."""

    end: int
    """
    The index of the character immediately after the last character of the answer in
    the text, starting from `0` (such that, in Python, the answer is equivalent to
    `text[start:end]`).
    """

    score: float
    """
    A score between `0` and `1`, inclusive, representing the strength of the answer.
    """

    start: int
    """
    The index of the first character of the answer in the text, starting from `0`
    (and, therefore, ending at the number of characters in the text minus `1`).
    """

    text: str
    """The text of the answer."""


class Extraction(BaseModel):
    """The result of extracting answers from a text."""

    answers: List[ExtractionAnswer]
    """Answers extracted from the text, ordered from highest to lowest score."""

    index: int
    """
    The index of the text in the input array of texts that this result represents,
    starting from `0` (and, therefore, ending at the number of texts minus `1`).
    """

    inextractability_score: float
    """
    A score between `0` and `1`, inclusive, representing the likelihood that an
    answer can not be extracted from the text.

    Where this score is greater than the highest score of all possible answers, the
    text should be regarded as not having an extractable answer to the query. If
    that is the case and `ignore_inextractability` is `false`, no answers will be
    returned.
    """


class Usage(BaseModel):
    """
    Statistics about the usage of resources in the process of extracting answers from the texts.
    """

    input_tokens: int
    """The number of tokens inputted to the model."""


class AnswerExtractionResponse(BaseModel):
    extractions: List[Extraction]
    """
    The results of extracting answers from the texts, ordered from highest to lowest
    answer confidence score (or else lowest to highest inextractability score if
    there are no answers for a text).
    """

    usage: Usage
    """
    Statistics about the usage of resources in the process of extracting answers
    from the texts.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["RerankingResponse", "Result", "Usage"]


class Result(BaseModel):
    index: int
    """
    The index of the text in the input array of texts, starting from `0` (and,
    therefore, ending at the number of texts minus `1`).
    """

    score: float
    """
    A score between `0` and `1`, inclusive, representing the relevance of the text
    to the query.
    """


class Usage(BaseModel):
    """Statistics about the usage of resources in the process of reranking the texts."""

    input_tokens: int
    """The number of tokens inputted to the model."""


class RerankingResponse(BaseModel):
    results: List[Result]
    """
    The rerankings of the texts, by relevance to the query, in order from highest to
    lowest relevance score.
    """

    usage: Usage
    """Statistics about the usage of resources in the process of reranking the texts."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .qa import (
    QaResource,
    AsyncQaResource,
    QaResourceWithRawResponse,
    AsyncQaResourceWithRawResponse,
    QaResourceWithStreamingResponse,
    AsyncQaResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ExtractionsResource", "AsyncExtractionsResource"]


class ExtractionsResource(SyncAPIResource):
    @cached_property
    def qa(self) -> QaResource:
        return QaResource(self._client)

    @cached_property
    def with_raw_response(self) -> ExtractionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return ExtractionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExtractionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return ExtractionsResourceWithStreamingResponse(self)


class AsyncExtractionsResource(AsyncAPIResource):
    @cached_property
    def qa(self) -> AsyncQaResource:
        return AsyncQaResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncExtractionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExtractionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExtractionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return AsyncExtractionsResourceWithStreamingResponse(self)


class ExtractionsResourceWithRawResponse:
    def __init__(self, extractions: ExtractionsResource) -> None:
        self._extractions = extractions

    @cached_property
    def qa(self) -> QaResourceWithRawResponse:
        return QaResourceWithRawResponse(self._extractions.qa)


class AsyncExtractionsResourceWithRawResponse:
    def __init__(self, extractions: AsyncExtractionsResource) -> None:
        self._extractions = extractions

    @cached_property
    def qa(self) -> AsyncQaResourceWithRawResponse:
        return AsyncQaResourceWithRawResponse(self._extractions.qa)


class ExtractionsResourceWithStreamingResponse:
    def __init__(self, extractions: ExtractionsResource) -> None:
        self._extractions = extractions

    @cached_property
    def qa(self) -> QaResourceWithStreamingResponse:
        return QaResourceWithStreamingResponse(self._extractions.qa)


class AsyncExtractionsResourceWithStreamingResponse:
    def __init__(self, extractions: AsyncExtractionsResource) -> None:
        self._extractions = extractions

    @cached_property
    def qa(self) -> AsyncQaResourceWithStreamingResponse:
        return AsyncQaResourceWithStreamingResponse(self._extractions.qa)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["EmbeddingResponse", "Embedding", "Usage"]


class Embedding(BaseModel):
    embedding: List[float]
    """The embedding of the content represented as an array of floating point numbers."""

    index: int
    """
    The position of the content in the input array of contents, starting from `0`
    (and, therefore, ending at the number of contents minus `1`).
    """


class Usage(BaseModel):
    """Statistics about the usage of resources in the process of embedding the inputs."""

    input_tokens: int
    """The number of tokens inputted to the model."""


class EmbeddingResponse(BaseModel):
    embeddings: List[Embedding]
    """The embeddings of the inputs."""

    usage: Usage
    """Statistics about the usage of resources in the process of embedding the inputs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .rerankings import (
    RerankingsResource,
    AsyncRerankingsResource,
    RerankingsResourceWithRawResponse,
    AsyncRerankingsResourceWithRawResponse,
    RerankingsResourceWithStreamingResponse,
    AsyncRerankingsResourceWithStreamingResponse,
)
from .enrichments import (
    EnrichmentsResource,
    AsyncEnrichmentsResource,
    EnrichmentsResourceWithRawResponse,
    AsyncEnrichmentsResourceWithRawResponse,
    EnrichmentsResourceWithStreamingResponse,
    AsyncEnrichmentsResourceWithStreamingResponse,
)
from .extractions import (
    ExtractionsResource,
    AsyncExtractionsResource,
    ExtractionsResourceWithRawResponse,
    AsyncExtractionsResourceWithRawResponse,
    ExtractionsResourceWithStreamingResponse,
    AsyncExtractionsResourceWithStreamingResponse,
)
from .classifications import (
    ClassificationsResource,
    AsyncClassificationsResource,
    ClassificationsResourceWithRawResponse,
    AsyncClassificationsResourceWithRawResponse,
    ClassificationsResourceWithStreamingResponse,
    AsyncClassificationsResourceWithStreamingResponse,
)

__all__ = [
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "ClassificationsResource",
    "AsyncClassificationsResource",
    "ClassificationsResourceWithRawResponse",
    "AsyncClassificationsResourceWithRawResponse",
    "ClassificationsResourceWithStreamingResponse",
    "AsyncClassificationsResourceWithStreamingResponse",
    "RerankingsResource",
    "AsyncRerankingsResource",
    "RerankingsResourceWithRawResponse",
    "AsyncRerankingsResourceWithRawResponse",
    "RerankingsResourceWithStreamingResponse",
    "AsyncRerankingsResourceWithStreamingResponse",
    "ExtractionsResource",
    "AsyncExtractionsResource",
    "ExtractionsResourceWithRawResponse",
    "AsyncExtractionsResourceWithRawResponse",
    "ExtractionsResourceWithStreamingResponse",
    "AsyncExtractionsResourceWithStreamingResponse",
    "EnrichmentsResource",
    "AsyncEnrichmentsResource",
    "EnrichmentsResourceWithRawResponse",
    "AsyncEnrichmentsResourceWithRawResponse",
    "EnrichmentsResourceWithStreamingResponse",
    "AsyncEnrichmentsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["QaCreateParams", "ChunkingOptions"]


class QaCreateParams(TypedDict, total=False):
    model: Required[Literal["kanon-answer-extractor", "kanon-answer-extractor-mini"]]
    """
    The ID of the
    [model](https://docs.isaacus.com/models#extractive-question-answering) to use
    for extractive question answering.
    """

    query: Required[str]
    """The query to extract the answer to.

    The query must contain at least one non-whitespace character.

    Unlike the texts from which the answer will be extracted, the query cannot be so
    long that it exceeds the maximum input length of the model.
    """

    texts: Required[SequenceNotStr[str]]
    """The texts to search for the answer in and extract the answer from.

    There must be at least one text.

    Each text must contain at least one non-whitespace character.
    """

    chunking_options: Optional[ChunkingOptions]
    """Options for how to split text into smaller chunks."""

    ignore_inextractability: bool
    """
    Whether to, if the model's score of the likelihood that an answer can not be
    extracted from a text is greater than the highest score of all possible answers,
    still return the highest scoring answers for that text.

    If you have already determined that the texts answer the query, for example, by
    using one of our classification or reranker models, then you should set this to
    `true`.
    """

    top_k: int
    """The number of highest scoring answers to return.

    If `null`, which is the default, all answers will be returned.
    """


class ChunkingOptions(TypedDict, total=False):
    """Options for how to split text into smaller chunks."""

    overlap_ratio: Optional[float]
    """A number greater than or equal to 0 and less than 1."""

    overlap_tokens: Optional[int]
    """A whole number greater than or equal to 0."""

    size: Optional[int]
    """A whole number greater than or equal to 1."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from isaacus import Isaacus, AsyncIsaacus
from tests.utils import assert_matches_type
from isaacus.types.extractions import AnswerExtractionResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestQa:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Isaacus) -> None:
        qa = client.extractions.qa.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
        )
        assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Isaacus) -> None:
        qa = client.extractions.qa.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
            chunking_options={
                "overlap_ratio": 0.1,
                "overlap_tokens": 10,
                "size": 512,
            },
            ignore_inextractability=False,
            top_k=1,
        )
        assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Isaacus) -> None:
        response = client.extractions.qa.with_raw_response.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        qa = response.parse()
        assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Isaacus) -> None:
        with client.extractions.qa.with_streaming_response.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            qa = response.parse()
            assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncQa:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncIsaacus) -> None:
        qa = await async_client.extractions.qa.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
        )
        assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncIsaacus) -> None:
        qa = await async_client.extractions.qa.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
            chunking_options={
                "overlap_ratio": 0.1,
                "overlap_tokens": 10,
                "size": 512,
            },
            ignore_inextractability=False,
            top_k=1,
        )
        assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncIsaacus) -> None:
        response = await async_client.extractions.qa.with_raw_response.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        qa = await response.parse()
        assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncIsaacus) -> None:
        async with async_client.extractions.qa.with_streaming_response.create(
            model="kanon-answer-extractor",
            query="What is the punishment for murder in Victoria?",
            texts=[
                "The standard sentence for murder in the State of Victoria is 30 years if the person murdered was a police officer and 25 years in any other case."
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            qa = await response.parse()
            assert_matches_type(AnswerExtractionResponse, qa, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["EnrichmentCreateParams"]


class EnrichmentCreateParams(TypedDict, total=False):
    model: Required[Literal["kanon-2-enricher-preview"]]
    """
    The ID of the [model](https://docs.isaacus.com/models#enrichment) to use for
    enrichment.
    """

    texts: Required[Union[SequenceNotStr[str], str]]
    """
    A text or array of texts to be enriched, each containing at least one
    non-whitespace character.

    No more than 8 texts can be enriched in a single request.
    """

    overflow_strategy: Optional[Literal["auto", "drop_end"]]
    """The strategy for handling content exceeding the model's maximum input length.

    `auto` currently behaves the same as `drop_end`, dropping excess tokens from the
    end of input. In the future, `auto` may implement more sophisticated strategies
    such as chunking and context-aware stitching.

    `drop_end` drops tokens from the end of input exceeding the model's maximum
    input length.

    `null`, which is the default setting, raises an error if the input exceeds the
    model's maximum input length.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal

import httpx

from ..types import embedding_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.embedding_response import EmbeddingResponse

__all__ = ["EmbeddingsResource", "AsyncEmbeddingsResource"]


class EmbeddingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return EmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return EmbeddingsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["kanon-2-embedder"],
        texts: Union[SequenceNotStr[str], str],
        dimensions: Optional[int] | Omit = omit,
        overflow_strategy: Optional[Literal["drop_end"]] | Omit = omit,
        task: Optional[Literal["retrieval/query", "retrieval/document"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingResponse:
        """
        Embed legal texts with an Isaacus legal AI embedder.

        Args:
          model: The ID of the [model](https://docs.isaacus.com/models#embedding) to use for
              embedding.

          texts: The text or array of texts to embed.

              Each text must contain at least one non-whitespace character.

              No more than 128 texts can be embedded in a single request.

          dimensions: A whole number greater than or equal to 1.

          overflow_strategy: The strategy to employ when content exceeds the model's maximum input length.

              `drop_end`, which is the default setting, drops tokens from the end of the
              content exceeding the limit.

              If `null`, an error will be raised if any content exceeds the model's maximum
              input length.

          task: The task the embeddings will be used for.

              `retrieval/query` is meant for queries and statements, and `retrieval/document`
              is meant for anything to be retrieved using query embeddings.

              If `null`, which is the default setting, embeddings will not be optimized for
              any particular task.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/embeddings",
            body=maybe_transform(
                {
                    "model": model,
                    "texts": texts,
                    "dimensions": dimensions,
                    "overflow_strategy": overflow_strategy,
                    "task": task,
                },
                embedding_create_params.EmbeddingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )


class AsyncEmbeddingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return AsyncEmbeddingsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["kanon-2-embedder"],
        texts: Union[SequenceNotStr[str], str],
        dimensions: Optional[int] | Omit = omit,
        overflow_strategy: Optional[Literal["drop_end"]] | Omit = omit,
        task: Optional[Literal["retrieval/query", "retrieval/document"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingResponse:
        """
        Embed legal texts with an Isaacus legal AI embedder.

        Args:
          model: The ID of the [model](https://docs.isaacus.com/models#embedding) to use for
              embedding.

          texts: The text or array of texts to embed.

              Each text must contain at least one non-whitespace character.

              No more than 128 texts can be embedded in a single request.

          dimensions: A whole number greater than or equal to 1.

          overflow_strategy: The strategy to employ when content exceeds the model's maximum input length.

              `drop_end`, which is the default setting, drops tokens from the end of the
              content exceeding the limit.

              If `null`, an error will be raised if any content exceeds the model's maximum
              input length.

          task: The task the embeddings will be used for.

              `retrieval/query` is meant for queries and statements, and `retrieval/document`
              is meant for anything to be retrieved using query embeddings.

              If `null`, which is the default setting, embeddings will not be optimized for
              any particular task.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/embeddings",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "texts": texts,
                    "dimensions": dimensions,
                    "overflow_strategy": overflow_strategy,
                    "task": task,
                },
                embedding_create_params.EmbeddingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )


class EmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = to_raw_response_wrapper(
            embeddings.create,
        )


class AsyncEmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = async_to_raw_response_wrapper(
            embeddings.create,
        )


class EmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = to_streamed_response_wrapper(
            embeddings.create,
        )


class AsyncEmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = async_to_streamed_response_wrapper(
            embeddings.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["UniversalClassificationResponse", "Classification", "ClassificationChunk", "Usage"]


class ClassificationChunk(BaseModel):
    end: int
    """
    The index of the character immediately after the last character of the chunk in
    the original text, beginning from `0` (such that, in Python, the chunk is
    equivalent to `text[start:end]`).
    """

    index: int
    """
    The original position of the chunk in the outputted list of chunks before
    sorting, starting from `0` (and, therefore, ending at the number of chunks minus
    `1`).
    """

    score: float
    """
    The model's score of the likelihood that the query expressed about the chunk is
    supported by the chunk.

    A score greater than `0.5` indicates that the chunk supports the query, while a
    score less than `0.5` indicates that the chunk does not support the query.
    """

    start: int
    """
    The index of the character in the original text where the chunk starts,
    beginning from `0`.
    """

    text: str
    """The text of the chunk."""


class Classification(BaseModel):
    chunks: Optional[List[ClassificationChunk]] = None
    """
    The text as broken into chunks by
    [semchunk](https://github.com/isaacus-dev/semchunk), each chunk with its own
    confidence score, ordered from highest to lowest score.

    If no chunking occurred, this will be `null`.
    """

    index: int
    """
    The index of the text in the input array of texts, starting from `0` (and,
    therefore, ending at the number of texts minus `1`).
    """

    score: float
    """
    A score of the likelihood that the query expressed about the text is supported
    by the text.

    A score greater than `0.5` indicates that the text supports the query, while a
    score less than `0.5` indicates that the text does not support the query.
    """


class Usage(BaseModel):
    """Statistics about the usage of resources in the process of classifying the text."""

    input_tokens: int
    """The number of tokens inputted to the model."""


class UniversalClassificationResponse(BaseModel):
    classifications: List[Classification]
    """
    The classifications of the texts, by relevance to the query, in order from
    highest to lowest relevance score.
    """

    usage: Usage
    """Statistics about the usage of resources in the process of classifying the text."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "EnrichmentResponse",
    "Result",
    "ResultDocument",
    "ResultDocumentCrossreference",
    "ResultDocumentCrossreferenceSpan",
    "ResultDocumentDate",
    "ResultDocumentDateMention",
    "ResultDocumentEmail",
    "ResultDocumentEmailMention",
    "ResultDocumentExternalDocument",
    "ResultDocumentExternalDocumentMention",
    "ResultDocumentExternalDocumentName",
    "ResultDocumentExternalDocumentPinpoint",
    "ResultDocumentHeading",
    "ResultDocumentIDNumber",
    "ResultDocumentIDNumberMention",
    "ResultDocumentJunk",
    "ResultDocumentLocation",
    "ResultDocumentLocationMention",
    "ResultDocumentLocationName",
    "ResultDocumentPerson",
    "ResultDocumentPersonMention",
    "ResultDocumentPersonName",
    "ResultDocumentPhoneNumber",
    "ResultDocumentPhoneNumberMention",
    "ResultDocumentQuote",
    "ResultDocumentQuoteSpan",
    "ResultDocumentSegment",
    "ResultDocumentSegmentCode",
    "ResultDocumentSegmentSpan",
    "ResultDocumentSegmentTitle",
    "ResultDocumentSegmentTypeName",
    "ResultDocumentSubtitle",
    "ResultDocumentTerm",
    "ResultDocumentTermMeaning",
    "ResultDocumentTermMention",
    "ResultDocumentTermName",
    "ResultDocumentTitle",
    "ResultDocumentWebsite",
    "ResultDocumentWebsiteMention",
    "Usage",
]


class ResultDocumentCrossreferenceSpan(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentCrossreference(BaseModel):
    """A cross-reference within the document pointing to one or more segments."""

    end: str
    """
    The unique identifier of the latest segment in the span of segments being
    cross-referenced with ties broken in favor of the least-nested (i.e., largest)
    segment. If the cross-reference points to a single segment, `start` and `end`
    will be identical.
    """

    span: ResultDocumentCrossreferenceSpan
    """The span of the segment within the document's text."""

    start: str
    """
    The unique identifier of the earliest segment in the span of segments being
    cross-referenced with ties broken in favor of the least-nested (i.e., largest)
    segment. If the cross-reference points to a single segment, `start` and `end`
    will be identical.
    """


class ResultDocumentDateMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentDate(BaseModel):
    """
    A date identified in a document belonging to one of the following types: `creation`, `signature`, `effective`, `expiry`, `delivery`, `renewal`, `payment`, `birth`, or `death`.

    Only Gregorian dates between the years 1000 and 9999 (inclusive) fitting into one of the supported date types are extractable.
    """

    mentions: List[ResultDocumentDateMention]
    """
    An array of one or more spans within the document's text where the date is
    mentioned.
    """

    person: Optional[str] = None
    """
    A unique identifier for a legal person in the format `per:{index}` where
    `{index}` is a non-negative incrementing integer starting from zero.
    """

    type: Literal["creation", "signature", "effective", "expiry", "delivery", "renewal", "payment", "birth", "death"]
    """
    The type of the date, being one of `creation`, `signature`, `effective`,
    `expiry`, `delivery`, `renewal`, `payment`, `birth`, or `death`. If a date is
    mentioned in a document that does not fit into a supported type, it will not be
    extracted.

    `creation` denotes the date the document was created. There may only be one
    `creation` date per document.

    `signature` denotes the date the document was signed.

    `effective` denotes the date when the document or a part thereof comes into
    effect (e.g., commencement or enactment dates).

    `expiry` denotes the date when the document or a part thereof is no longer in
    effect.

    `delivery` denotes the date when goods or services are to be delivered under the
    document.

    `renewal` denotes the date when one or more of the document's terms are to be
    renewed.

    `payment` denotes the date when payment is to be made under the document.

    `birth` denotes the birth date of a natural person or establishment (e.g.,
    incorporation) date of a non-natural legal person identified in the document.
    There can only be one `birth` date linked to a single person and all `birth`
    dates must be linked to a person. A person's `birth` date will never be after
    their `death` date.

    `death` denotes the death date of a natural person or dissolution date of a
    non-natural legal person identified in the document. There can only be one
    `death` date linked to a single person and all `death` dates must be linked to a
    person. A person's `death` date will never be before their `birth` date.
    """

    value: str
    """The date in ISO 8601 format (YYYY-MM-DD)."""


class ResultDocumentEmailMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentEmail(BaseModel):
    """An email address identified in a document belonging to a legal person.

    If an email address was mentioned in the document but is not attributable to a legal person, it will not be extracted.
    """

    address: str
    """The normalized email address."""

    mentions: List[ResultDocumentEmailMention]
    """
    An array of one or more spans within the document's text where the email address
    is mentioned.
    """

    person: str
    """The unique identifier of the person that this email address belongs to."""


class ResultDocumentExternalDocumentMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentExternalDocumentName(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentExternalDocumentPinpoint(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentExternalDocument(BaseModel):
    """A document identified within another document."""

    id: str
    """
    The unique identifier of the external document in the format `exd:{index}` where
    `{index}` is a non-negative incrementing integer starting from zero.
    """

    jurisdiction: Optional[str] = None
    """
    A jurisdiction code representing a country (via an initial country code) and,
    optionally, a subdivision within that country (via a subsequent subdivision code
    prefixed by a hyphen).

    All 249 ISO 3166-1 alpha-2 country codes are representable in addition to
    special `INT` and `EU` codes for international and European Union law,
    respectively.

    All 5,046 ISO 3166-2 codes are also representable in addition to a special `FED`
    code for federal law.
    """

    mentions: List[ResultDocumentExternalDocumentMention]
    """
    An array of one or more spans within the document's text where the external
    document is mentioned by name, for example, 'the US Constitution' in 'the Second
    Amendment to the US Constitution protects freedom of speech'.
    """

    name: ResultDocumentExternalDocumentName
    """The span of the segment within the document's text."""

    pinpoints: List[ResultDocumentExternalDocumentPinpoint]
    """
    An array of spans within the document's text where specific parts of the
    external document are referenced, for example, 'Section 2' in 'as defined in
    Section 2 of the US Constitution'.
    """

    reception: Literal["positive", "mixed", "negative", "neutral"]
    """
    The sentiment of the document towards the external document, being one of
    `positive`, `mixed`, `negative`, or `neutral`.

    `positive` indicates that the document expresses a favorable view of the
    external document whether by endorsing or approving it.

    `mixed` indicates that the document expresses both favorable and unfavorable
    views of the external document, for example, by affirming parts of it and
    disapproving others.

    `negative` indicates that the document expresses an unfavorable view of the
    external document whether by criticizing, repealing, overruling, or explicitly
    contradicting it.

    `neutral` indicates that the document references the external document without
    expressing any particular sentiment towards it.
    """

    type: Literal["statute", "regulation", "decision", "contract", "other"]
    """
    The type of the external document, being one of `statute`, `regulation`,
    `decision`, `contract`, or `other`.

    `statute` denotes primary legislation such as acts, bills, codes, and
    constitutions.

    `regulation` denotes secondary legislation such as rules, statutory instruments,
    and ordinances.

    `decision` denotes judicial or quasi-judicial decisions such as court judgments,
    judicial opinions, and tribunal rulings.

    `other` is used for all other types of legal documents that do not fit into any
    of the predefined types.
    """


class ResultDocumentHeading(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentIDNumberMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentIDNumber(BaseModel):
    """An identification number mentioned in a document belonging to a legal person.

    If an identification number was mentioned in the document but is not attributable to a legal person, it will not be extracted.
    """

    mentions: List[ResultDocumentIDNumberMention]
    """
    An array of one or more spans within the document's text where the
    identification number is mentioned.
    """

    number: str
    """The identification number."""

    person: str
    """The unique identifier of the person that this identification number belongs to."""


class ResultDocumentJunk(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentLocationMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentLocationName(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentLocation(BaseModel):
    """A location identified within a document."""

    id: str
    """
    The unique identifier of the location in the format `loc:{index}` where
    `{index}` is a non-negative incrementing integer starting from zero.
    """

    mentions: List[ResultDocumentLocationMention]
    """
    An array of one or more spans within the document's text where the location is
    mentioned.
    """

    name: ResultDocumentLocationName
    """The span of the segment within the document's text."""

    parent: Optional[str] = None
    """
    A unique identifier for a location in the format `loc:{index}` where `{index}`
    is a non-negative incrementing integer starting from zero.
    """

    type: Literal["country", "state", "city", "address", "other"]
    """
    The type of the location, being one of `country`, `state`, `city`, `address`, or
    `other`.
    """


class ResultDocumentPersonMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentPersonName(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentPerson(BaseModel):
    """A legal person identified in a document."""

    id: str
    """
    The unique identifier of the person in the format `per:{index}` where `{index}`
    is a non-negative incrementing integer starting from zero.
    """

    mentions: List[ResultDocumentPersonMention]
    """
    An array of one or more spans within the document's text where the person is
    mentioned.
    """

    name: ResultDocumentPersonName
    """The span of the segment within the document's text."""

    parent: Optional[str] = None
    """
    A unique identifier for a legal person in the format `per:{index}` where
    `{index}` is a non-negative incrementing integer starting from zero.
    """

    residence: Optional[str] = None
    """
    A unique identifier for a location in the format `loc:{index}` where `{index}`
    is a non-negative incrementing integer starting from zero.
    """

    role: Literal[
        "plaintiff",
        "petitioner",
        "applicant",
        "appellant",
        "appellee",
        "claimant",
        "complainant",
        "defendant",
        "respondent",
        "prior_authority",
        "prosecutor",
        "defense_counsel",
        "amicus",
        "intervener",
        "borrower",
        "lender",
        "guarantor",
        "lessee",
        "lessor",
        "employer",
        "employee",
        "licensor",
        "licensee",
        "franchisor",
        "franchisee",
        "buyer",
        "seller",
        "contractor",
        "shareholder",
        "joint_venturer",
        "investor",
        "insurer",
        "insured",
        "enacting_authority",
        "empowered_authority",
        "settlor",
        "trustee",
        "beneficiary",
        "debater",
        "director",
        "governing_jurisdiction",
        "clerk",
        "witness",
        "other",
        "non_party",
    ]
    """The role of the person in relation to the subject of the document.

    The following roles are currently supported: | | | | ------------------------ |
    ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    | | `plaintiff` | A party initiating the case that is the subject of the
    document. | | `petitioner` | A party initiating the petition that is the subject
    of the document. | | `applicant` | A party initiating the application that is
    the subject of the document. | | `appellant` | A party appealing the decision
    that is the subject of the document. | | `appellee` | A party responding to the
    appeal that is the subject of the document if they are explicitly referred to as
    an 'appellee'. | | `claimant` | A party making a claim in the case that is the
    subject of the document. | | `complainant` | A party making a complaint in the
    case that is the subject of the document. | | `defendant` | A party defending
    against the case that is the subject of the document. | | `respondent` | A party
    responding to the petition, appeal, or application that is the subject of the
    document. | | `prior_authority` | An authority (e.g., judge, tribunal, court)
    that made a prior decision in the case that is the subject of the document. Both
    individual judges and courts should be annotated with this role where
    applicable. This is not to be used for authorities cited as precedent, only for
    those that made prior decisions in the same case. | | `prosecutor` | A lawyer
    prosecuting the case that is the subject of the document. | | `defense_counsel`
    | A lawyer defending the case that is the subject of the document. | | `amicus`
    | A party filing an amicus curiae brief in the case that is the subject of the
    document. | | `intervener` | A party attempting to or that has intervened in the
    case that is the subject of the document. | | `borrower` | A party borrowing
    money or other assets under the agreement that is the subject of the document,
    including 'mortgagors' and 'debtors'. | | `lender` | A party lending money or
    other assets under the agreement that is the subject of the document, including
    'mortgagees' and 'creditors'. | | `guarantor` | A party guaranteeing obligations
    under the agreement that is the subject of the document, including 'sureties'. |
    | `lessee` | A party leasing goods or services under the agreement that is the
    subject of the document, including 'tenants'. | | `lessor` | A party leasing
    goods or services under the agreement that is the subject of the document,
    including 'landlords'. | | `employer` | A party employing personnel under the
    agreement that is the subject of the document. | | `employee` | A party employed
    under the agreement that is the subject of the document. | | `licensor` | A
    party licensing intellectual property or other rights under the agreement that
    are the subject of the document. | | `licensee` | A party licensed to use
    intellectual property or other rights under the agreement that are the subject
    of the document. | | `franchisor` | A party granting a franchise under the
    agreement that is the subject of the document. | | `franchisee` | A party
    granted a franchise under the agreement that is the subject of the document. | |
    `buyer` | A party purchasing goods or services under the agreement that is the
    subject of the document, including 'purchasers', 'customers', and 'clients'. | |
    `seller` | A party selling or providing goods or services under the agreement
    that is the subject of the document, including 'Vendors', 'Suppliers', and
    'Service Providers' (where such parties are actually providing goods or services
    under the agreement). | | `contractor` | A party contracted to perform work or
    services under the agreement that is the subject of the document, including
    'consultants'. | | `shareholder` | A party holding shares or equity under the
    agreement that is the subject of the document. | | `joint_venturer` | A party
    participating in a joint venture under the agreement that is the subject of the
    document. | | `investor` | A party investing money or assets under the agreement
    that is the subject of the document. | | `insurer` | A party providing insurance
    under the agreement that is the subject of the document. | | `insured` | A party
    insured under the agreement that is the subject of the document. | | `settlor` |
    A party establishing the trust that is the subject of the document. | |
    `trustee` | A party managing the trust that is the subject of the document. | |
    `beneficiary` | A party benefiting from the trust that is the subject of the
    document. | | `enacting_authority` | An authority (e.g., legislature, regulator,
    Minister/Secretary, President/Prime Minister, tribunal, court, judge) giving
    legal effect to or authorizing the document. All relevant individuals and bodies
    should be annotated with this role where applicable. | | `empowered_authority` |
    An authority (e.g., government agency, regulator, Minister/Secretary,
    President/Prime Minister, tribunal, court) empowered by the document to carry
    out functions or duties. | | `debater` | A person participating in the debate
    that is the subject of the document. | | `governing_jurisdiction` | The
    jurisdiction whose laws govern the document. | | `director` | A director or
    other officer of a corporate legal person mentioned in the document. | | `clerk`
    | A clerk, notary, or other official certifying, witnessing, filing, recording,
    registering, or otherwise administering the document. | | `witness` | A witness
    witnessing the signing of the document, or whose testimony is part of the case
    that is the subject of the document. | | `other` | A party to the case,
    agreement, legislation, or regulation that is the subject of the document that
    does not fit into any of the other roles. | | `non_party` | A legal person
    mentioned in the document that is not a party to the case, agreement,
    legislation, or regulation that is the subject of the document. |
    """

    type: Literal["natural", "corporate", "politic"]
    """
    The legal entity type of the person, being one of `natural`, `corporate`, or
    `politic`.

    `natural` denotes a human being in their capacity as a natural legal person,
    including when representing unincorporated entities such as partnerships and
    trusts.

    `corporate` denotes a body corporate such as a company, incorporated
    partnership, or statutory corporation.

    `politic` denotes a body politic such as a court, state, government, or
    intergovernmental organization.
    """


class ResultDocumentPhoneNumberMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentPhoneNumber(BaseModel):
    """A valid phone number identified in a document belonging to a legal person.

    If a phone number was mentioned in the document but is not valid, possible, or attributable to a legal person, it will not be extracted.
    """

    mentions: List[ResultDocumentPhoneNumberMention]
    """
    An array of one or more spans within the document's text where the phone number
    is mentioned.
    """

    number: str
    """
    The normalized phone number in E.123 international notation conforming with
    local conventions on the use of spaces and hyphens as separators.
    """

    person: str
    """The unique identifier of the person that this phone number belongs to."""


class ResultDocumentQuoteSpan(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentQuote(BaseModel):
    """A quotation within a document."""

    amending: bool
    """
    Whether the quote is being used to amend or modify content, typically in other
    documents.
    """

    source_document: Optional[str] = None
    """
    A unique identifier for an external document in the format `exd:{index}` where
    `{index}` is a non-negative incrementing integer starting from zero.
    """

    source_person: Optional[str] = None
    """
    A unique identifier for a legal person in the format `per:{index}` where
    `{index}` is a non-negative incrementing integer starting from zero.
    """

    source_segment: Optional[str] = None
    """
    A unique identifier for a segment in the format `seg:{index}` where `{index}` is
    a non-negative incrementing integer starting from zero.
    """

    span: ResultDocumentQuoteSpan
    """The span of the segment within the document's text."""


class ResultDocumentSegmentCode(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentSegmentSpan(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentSegmentTitle(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentSegmentTypeName(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentSegment(BaseModel):
    """
    A segment within the document representing a structurally distinct portion of the document's content.
    """

    id: str
    """
    The unique identifier of the segment in the format `seg:{index}` where `{index}`
    is a non-negative incrementing integer starting from zero.
    """

    category: Literal["front_matter", "scope", "main", "annotation", "back_matter", "other"]
    """
    The functional 'category' of the segment within the document, being one of
    `front_matter`, `scope`, `main`, `annotation`, `back_matter`, or `other`.

    `front_matter` denotes non-operative contextualizing content occurring at the
    start of a document such as a preamble or recitals.

    `scope` denotes operative content defining the application or interpretation of
    a document such as definition sections and governing law clauses.

    `main` denotes operative, non-scopal content.

    `annotation` denotes non-operative annotative content providing explanatory or
    referential information such as commentary, footnotes, and endnotes.

    `back_matter` denotes non-operative contextualizing content occurring at the end
    of a document such as authority statements.

    `other` denotes content that does not fit into any of the other categories.
    """

    code: Optional[ResultDocumentSegmentCode] = None
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible
    for any two spans to partially overlap; they can only be disjoint, adjacent, or
    wholly nested. Spans of the exact same type (e.g., segments) will never be
    duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses
    zero-based, half-open, Unicode code point-spaced string indexing), indices may
    need to be translated accordingly (for example, JavaScript slices into UTF-16
    code units instead of Unicode code points).
    """

    kind: Literal["container", "unit", "item", "figure"]
    """
    The structural 'kind' of the segment, being one of `container`, `unit`, `item`,
    or `figure`.

    A `container` is a structural or semantic grouping of content such as a chapter.
    It can contain segments of any kind or none at all.

    A `unit` is a single syntactically independent unit of text such as a paragraph.
    It can only contain `item`s and `figure`s.

    An `item` is a syntactically subordinate unit of text such as an item in a
    run-in list. It can only contain other `item`s. Note that an `item` is
    conceptually distinct from a list item—it is perfectly possible to encounter
    list items that are syntactically independent of their surrounding items just as
    it is possible to encounter dependent clauses that do not appear as part of a
    list.

    A `figure` is a visually structured or tabular unit of content such as a
    diagram, equation, or table. It cannot contain segments.
    """

    parent: Optional[str] = None
    """
    A unique identifier for a segment in the format `seg:{index}` where `{index}` is
    a non-negative incrementing integer starting from zero.
    """

    span: ResultDocumentSegmentSpan
    """The span of the segment within the document's text."""

    title: Optional[ResultDocumentSegmentTitle] = None
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible
    for any two spans to partially overlap; they can only be disjoint, adjacent, or
    wholly nested. Spans of the exact same type (e.g., segments) will never be
    duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses
    zero-based, half-open, Unicode code point-spaced string indexing), indices may
    need to be translated accordingly (for example, JavaScript slices into UTF-16
    code units instead of Unicode code points).
    """

    type: Optional[
        Literal[
            "title",
            "book",
            "part",
            "chapter",
            "subchapter",
            "division",
            "subdivision",
            "subpart",
            "subtitle",
            "table_of_contents",
            "article",
            "section",
            "regulation",
            "rule",
            "clause",
            "paragraph",
            "subarticle",
            "subsection",
            "subregulation",
            "subrule",
            "subclause",
            "subparagraph",
            "item",
            "subitem",
            "point",
            "indent",
            "schedule",
            "annex",
            "appendix",
            "exhibit",
            "recital",
            "signature",
            "note",
            "figure",
            "table",
            "formula",
        ]
    ] = None
    """
    The addressable 'type' of the segment within the document's referential scheme
    and hierarchy, whether defined explicitly (e.g., by headings, such as
    'Section 2. Definitions'), implicitly (e.g., by way of reference, such as 'as
    defined in Section 2'), or by convention (e.g., [42] in a judgment often denotes
    a `paragraph`, independent provisions in statute are often `section`s, etc.). If
    the type is not known or not applicable, it will be set to `null`.

    Note that, although many segment types may coincide with syntactic constructs,
    they should be thought of purely as distinct formal citable units. Most
    paragraphs (in the syntactic sense) will not have the `paragraph` type, for
    example. That type is reserved for segments that would formally be cited as a
    'Paragraph' within the document's referential scheme.

    The following types are currently supported: `title`, `book`, `part`, `chapter`,
    `subchapter`, `division`, `subdivision`, `subpart`, `subtitle`,
    `table_of_contents`, `article`, `section`, `regulation`, `rule`, `clause`,
    `paragraph`, `subarticle`, `subsection`, `subregulation`, `subrule`,
    `subclause`, `subparagraph`, `item`, `subitem`, `point`, `indent`, `schedule`,
    `annex`, `appendix`, `exhibit`, `recital`, `signature`, `note`, `figure`,
    `table`, and `formula`.

    The `title`, `book`, `part`, `chapter`, `subchapter`, `division`, `subdivision`,
    `subpart`, `subtitle`, and `table_of_contents` types are exclusive to the
    `container` kind.

    The `figure` kind only supports the `figure`, `table`, and `formula` types, all
    of which are exclusive to it.
    """

    type_name: Optional[ResultDocumentSegmentTypeName] = None
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible
    for any two spans to partially overlap; they can only be disjoint, adjacent, or
    wholly nested. Spans of the exact same type (e.g., segments) will never be
    duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses
    zero-based, half-open, Unicode code point-spaced string indexing), indices may
    need to be translated accordingly (for example, JavaScript slices into UTF-16
    code units instead of Unicode code points).
    """


class ResultDocumentSubtitle(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentTermMeaning(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentTermMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentTermName(BaseModel):
    """The span of the segment within the document's text."""

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentTerm(BaseModel):
    """A term assigned a definite meaning within a document."""

    id: str
    """
    The unique identifier of the term in the format `term:{index}` where `{index}`
    is a non-negative incrementing integer starting from zero.
    """

    meaning: ResultDocumentTermMeaning
    """The span of the segment within the document's text."""

    mentions: List[ResultDocumentTermMention]
    """
    An array of spans within the document's text where the term is mentioned outside
    of its definition.

    It is possible for the term to have no mentions if, outside of its definition,
    it is never referred to in the document.
    """

    name: ResultDocumentTermName
    """The span of the segment within the document's text."""


class ResultDocumentTitle(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentWebsiteMention(BaseModel):
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible for any two spans to partially overlap; they can only be disjoint, adjacent, or wholly nested. Spans of the exact same type (e.g., segments) will never be duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses zero-based, half-open, Unicode code point-spaced string indexing), indices may need to be translated accordingly (for example, JavaScript slices into UTF-16 code units instead of Unicode code points).
    """

    end: int
    """
    The zero-based end index of the half-open span (i.e., the end is exclusive) of
    Unicode code points in the input text.
    """

    start: int
    """
    The zero-based start index of the half-open span of Unicode code points in the
    input text.
    """


class ResultDocumentWebsite(BaseModel):
    """A website identified in a document belonging to a legal person.

    If a website was mentioned in the document but is not attributable to a legal person, it will not be extracted.
    """

    mentions: List[ResultDocumentWebsiteMention]
    """
    An array of one or more spans within the document's text where the website is
    mentioned (including paths and slugs which are not part of the website's
    normalized URL).
    """

    person: str
    """The unique identifier of the person that this website belongs to."""

    url: str
    """The normalized URL of the website in the form `https://{host}/`."""


class ResultDocument(BaseModel):
    """The enriched document."""

    crossreferences: List[ResultDocumentCrossreference]
    """
    An array of cross-references within the document pointing to a single segment or
    a span of segments.
    """

    dates: List[ResultDocumentDate]
    """
    An array of dates identified in the document belonging to one of the following
    types: `creation`, `signature`, `effective`, `expiry`, `delivery`, `renewal`,
    `payment`, `birth`, or `death`.

    Only Gregorian dates between the years 1000 and 9999 (inclusive) fitting into
    one of the supported date types are extractable.
    """

    emails: List[ResultDocumentEmail]
    """
    An array of email addresses identified in the document belonging to legal
    persons.

    Email addresses mentioned in the document that are not attributable to legal
    persons will not be extracted.
    """

    external_documents: List[ResultDocumentExternalDocument]
    """An array of documents identified within the document."""

    headings: List[ResultDocumentHeading]
    """An array of spans within the document's text constituting headings."""

    id_numbers: List[ResultDocumentIDNumber]
    """
    An array of identification numbers identified in the document belonging to legal
    persons.

    Identification numbers mentioned in the document that are not attributable to
    legal persons will not be extracted.
    """

    junk: List[ResultDocumentJunk]
    """
    An array of spans within the document's text constituting non-operative,
    non-substantive 'junk' content such as headers, footers, page numbers, and OCR
    artifacts.
    """

    jurisdiction: Optional[str] = None
    """
    A jurisdiction code representing a country (via an initial country code) and,
    optionally, a subdivision within that country (via a subsequent subdivision code
    prefixed by a hyphen).

    All 249 ISO 3166-1 alpha-2 country codes are representable in addition to
    special `INT` and `EU` codes for international and European Union law,
    respectively.

    All 5,046 ISO 3166-2 codes are also representable in addition to a special `FED`
    code for federal law.
    """

    locations: List[ResultDocumentLocation]
    """An array of locations identified in the document."""

    persons: List[ResultDocumentPerson]
    """An array of legal persons identified in the document."""

    phone_numbers: List[ResultDocumentPhoneNumber]
    """
    An array of valid phone numbers identified in the document belonging to legal
    persons.

    Phone numbers mentioned in the document that are not valid, possible, or
    attributable to legal persons will not be extracted.
    """

    quotes: List[ResultDocumentQuote]
    """An array of quotations within the document."""

    segments: List[ResultDocumentSegment]
    """
    An array of segments within the document representing structurally distinct
    portions of its content.
    """

    subtitle: Optional[ResultDocumentSubtitle] = None
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible
    for any two spans to partially overlap; they can only be disjoint, adjacent, or
    wholly nested. Spans of the exact same type (e.g., segments) will never be
    duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses
    zero-based, half-open, Unicode code point-spaced string indexing), indices may
    need to be translated accordingly (for example, JavaScript slices into UTF-16
    code units instead of Unicode code points).
    """

    terms: List[ResultDocumentTerm]
    """An array of terms assigned definite meanings within the document."""

    title: Optional[ResultDocumentTitle] = None
    """A zero-based, half-open span into the Unicode code point space of input text.

    All spans are globally laminar and well-nested similar to XML—it is impossible
    for any two spans to partially overlap; they can only be disjoint, adjacent, or
    wholly nested. Spans of the exact same type (e.g., segments) will never be
    duplicated.

    A span cannot be empty and will never start or end at whitespace.

    Note that, when using programming languages other than Python (which uses
    zero-based, half-open, Unicode code point-spaced string indexing), indices may
    need to be translated accordingly (for example, JavaScript slices into UTF-16
    code units instead of Unicode code points).
    """

    type: Literal["statute", "regulation", "decision", "contract", "other"]
    """
    The type of the document, being one of `statute`, `regulation`, `decision`,
    `contract`, or `other`.

    `statute` denotes primary legislation such as acts, bills, codes, and
    constitutions.

    `regulation` denotes secondary legislation such as rules, statutory instruments,
    and ordinances.

    `decision` denotes judicial or quasi-judicial decisions such as court judgments,
    judicial opinions, and tribunal rulings.

    `other` is used for all other types of legal documents that do not fit into any
    of the predefined types.
    """

    version: Literal["ilgs@1"]

    websites: List[ResultDocumentWebsite]
    """An array of websites identified in the document belonging to legal persons.

    Websites mentioned in the document that are not attributable to legal persons
    will not be extracted.
    """


class Result(BaseModel):
    """An enriched document alongside its index in the input array of texts."""

    document: ResultDocument
    """The enriched document."""

    index: int
    """
    The index of this document in the input array of texts, starting at `0` (and,
    therefore, ending at the number of inputs minus `1`).
    """


class Usage(BaseModel):
    """Statistics about the usage of resources in the process of enriching the input."""

    input_tokens: int
    """The total number of tokens inputted to the model."""


class EnrichmentResponse(BaseModel):
    results: List[Result]
    """
    The input documents enriched into version 1.0.0 of the Isaacus Legal Graph
    Schema (IGLS).

    All spans in an enriched document graph are indexed into the Unicode code point
    space of a source document. Access to source documents is thus required to
    resolve spans into text.

    The start and end indices of spans are zero-based (i.e., the first Unicode code
    point in the document is at index 0) and half-open (i.e., the end index is
    exclusive).

    All spans are globally laminar and well-nested similar to XML—it is impossible
    for any two spans to partially overlap; they can only be disjoint, adjacent, or
    wholly nested.

    Spans of the exact same type (e.g., segments) will never be duplicated.

    Spans cannot be empty and will never start or end at whitespace.

    When using programming languages other than Python (which uses zero-based,
    half-open, Unicode code point-spaced string indexing), indices may need to be
    translated accordingly (for example, JavaScript slices into UTF-16 code units
    instead of Unicode code points).
    """

    usage: Usage
    """Statistics about the usage of resources in the process of enriching the input."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.extractions import qa_create_params
from ...types.extractions.answer_extraction_response import AnswerExtractionResponse

__all__ = ["QaResource", "AsyncQaResource"]


class QaResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return QaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return QaResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["kanon-answer-extractor", "kanon-answer-extractor-mini"],
        query: str,
        texts: SequenceNotStr[str],
        chunking_options: Optional[qa_create_params.ChunkingOptions] | Omit = omit,
        ignore_inextractability: bool | Omit = omit,
        top_k: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AnswerExtractionResponse:
        """
        Extract answers to questions from legal documents with an Isaacus legal AI
        answer extractor.

        Args:
          model: The ID of the
              [model](https://docs.isaacus.com/models#extractive-question-answering) to use
              for extractive question answering.

          query: The query to extract the answer to.

              The query must contain at least one non-whitespace character.

              Unlike the texts from which the answer will be extracted, the query cannot be so
              long that it exceeds the maximum input length of the model.

          texts: The texts to search for the answer in and extract the answer from.

              There must be at least one text.

              Each text must contain at least one non-whitespace character.

          chunking_options: Options for how to split text into smaller chunks.

          ignore_inextractability: Whether to, if the model's score of the likelihood that an answer can not be
              extracted from a text is greater than the highest score of all possible answers,
              still return the highest scoring answers for that text.

              If you have already determined that the texts answer the query, for example, by
              using one of our classification or reranker models, then you should set this to
              `true`.

          top_k: The number of highest scoring answers to return.

              If `null`, which is the default, all answers will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/extractions/qa",
            body=maybe_transform(
                {
                    "model": model,
                    "query": query,
                    "texts": texts,
                    "chunking_options": chunking_options,
                    "ignore_inextractability": ignore_inextractability,
                    "top_k": top_k,
                },
                qa_create_params.QaCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AnswerExtractionResponse,
        )


class AsyncQaResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return AsyncQaResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["kanon-answer-extractor", "kanon-answer-extractor-mini"],
        query: str,
        texts: SequenceNotStr[str],
        chunking_options: Optional[qa_create_params.ChunkingOptions] | Omit = omit,
        ignore_inextractability: bool | Omit = omit,
        top_k: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AnswerExtractionResponse:
        """
        Extract answers to questions from legal documents with an Isaacus legal AI
        answer extractor.

        Args:
          model: The ID of the
              [model](https://docs.isaacus.com/models#extractive-question-answering) to use
              for extractive question answering.

          query: The query to extract the answer to.

              The query must contain at least one non-whitespace character.

              Unlike the texts from which the answer will be extracted, the query cannot be so
              long that it exceeds the maximum input length of the model.

          texts: The texts to search for the answer in and extract the answer from.

              There must be at least one text.

              Each text must contain at least one non-whitespace character.

          chunking_options: Options for how to split text into smaller chunks.

          ignore_inextractability: Whether to, if the model's score of the likelihood that an answer can not be
              extracted from a text is greater than the highest score of all possible answers,
              still return the highest scoring answers for that text.

              If you have already determined that the texts answer the query, for example, by
              using one of our classification or reranker models, then you should set this to
              `true`.

          top_k: The number of highest scoring answers to return.

              If `null`, which is the default, all answers will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/extractions/qa",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "query": query,
                    "texts": texts,
                    "chunking_options": chunking_options,
                    "ignore_inextractability": ignore_inextractability,
                    "top_k": top_k,
                },
                qa_create_params.QaCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AnswerExtractionResponse,
        )


class QaResourceWithRawResponse:
    def __init__(self, qa: QaResource) -> None:
        self._qa = qa

        self.create = to_raw_response_wrapper(
            qa.create,
        )


class AsyncQaResourceWithRawResponse:
    def __init__(self, qa: AsyncQaResource) -> None:
        self._qa = qa

        self.create = async_to_raw_response_wrapper(
            qa.create,
        )


class QaResourceWithStreamingResponse:
    def __init__(self, qa: QaResource) -> None:
        self._qa = qa

        self.create = to_streamed_response_wrapper(
            qa.create,
        )


class AsyncQaResourceWithStreamingResponse:
    def __init__(self, qa: AsyncQaResource) -> None:
        self._qa = qa

        self.create = async_to_streamed_response_wrapper(
            qa.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal

import httpx

from ..types import enrichment_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.enrichment_response import EnrichmentResponse

__all__ = ["EnrichmentsResource", "AsyncEnrichmentsResource"]


class EnrichmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EnrichmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return EnrichmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EnrichmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return EnrichmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["kanon-2-enricher-preview"],
        texts: Union[SequenceNotStr[str], str],
        overflow_strategy: Optional[Literal["auto", "drop_end"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnrichmentResponse:
        """
        Enrich documents with an Isaacus enricher model.

        Args:
          model: The ID of the [model](https://docs.isaacus.com/models#enrichment) to use for
              enrichment.

          texts: A text or array of texts to be enriched, each containing at least one
              non-whitespace character.

              No more than 8 texts can be enriched in a single request.

          overflow_strategy: The strategy for handling content exceeding the model's maximum input length.

              `auto` currently behaves the same as `drop_end`, dropping excess tokens from the
              end of input. In the future, `auto` may implement more sophisticated strategies
              such as chunking and context-aware stitching.

              `drop_end` drops tokens from the end of input exceeding the model's maximum
              input length.

              `null`, which is the default setting, raises an error if the input exceeds the
              model's maximum input length.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/enrichments",
            body=maybe_transform(
                {
                    "model": model,
                    "texts": texts,
                    "overflow_strategy": overflow_strategy,
                },
                enrichment_create_params.EnrichmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EnrichmentResponse,
        )


class AsyncEnrichmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEnrichmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEnrichmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEnrichmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return AsyncEnrichmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["kanon-2-enricher-preview"],
        texts: Union[SequenceNotStr[str], str],
        overflow_strategy: Optional[Literal["auto", "drop_end"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnrichmentResponse:
        """
        Enrich documents with an Isaacus enricher model.

        Args:
          model: The ID of the [model](https://docs.isaacus.com/models#enrichment) to use for
              enrichment.

          texts: A text or array of texts to be enriched, each containing at least one
              non-whitespace character.

              No more than 8 texts can be enriched in a single request.

          overflow_strategy: The strategy for handling content exceeding the model's maximum input length.

              `auto` currently behaves the same as `drop_end`, dropping excess tokens from the
              end of input. In the future, `auto` may implement more sophisticated strategies
              such as chunking and context-aware stitching.

              `drop_end` drops tokens from the end of input exceeding the model's maximum
              input length.

              `null`, which is the default setting, raises an error if the input exceeds the
              model's maximum input length.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/enrichments",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "texts": texts,
                    "overflow_strategy": overflow_strategy,
                },
                enrichment_create_params.EnrichmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EnrichmentResponse,
        )


class EnrichmentsResourceWithRawResponse:
    def __init__(self, enrichments: EnrichmentsResource) -> None:
        self._enrichments = enrichments

        self.create = to_raw_response_wrapper(
            enrichments.create,
        )


class AsyncEnrichmentsResourceWithRawResponse:
    def __init__(self, enrichments: AsyncEnrichmentsResource) -> None:
        self._enrichments = enrichments

        self.create = async_to_raw_response_wrapper(
            enrichments.create,
        )


class EnrichmentsResourceWithStreamingResponse:
    def __init__(self, enrichments: EnrichmentsResource) -> None:
        self._enrichments = enrichments

        self.create = to_streamed_response_wrapper(
            enrichments.create,
        )


class AsyncEnrichmentsResourceWithStreamingResponse:
    def __init__(self, enrichments: AsyncEnrichmentsResource) -> None:
        self._enrichments = enrichments

        self.create = async_to_streamed_response_wrapper(
            enrichments.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .universal import (
    UniversalResource,
    AsyncUniversalResource,
    UniversalResourceWithRawResponse,
    AsyncUniversalResourceWithRawResponse,
    UniversalResourceWithStreamingResponse,
    AsyncUniversalResourceWithStreamingResponse,
)
from .classifications import (
    ClassificationsResource,
    AsyncClassificationsResource,
    ClassificationsResourceWithRawResponse,
    AsyncClassificationsResourceWithRawResponse,
    ClassificationsResourceWithStreamingResponse,
    AsyncClassificationsResourceWithStreamingResponse,
)

__all__ = [
    "UniversalResource",
    "AsyncUniversalResource",
    "UniversalResourceWithRawResponse",
    "AsyncUniversalResourceWithRawResponse",
    "UniversalResourceWithStreamingResponse",
    "AsyncUniversalResourceWithStreamingResponse",
    "ClassificationsResource",
    "AsyncClassificationsResource",
    "ClassificationsResourceWithRawResponse",
    "AsyncClassificationsResourceWithRawResponse",
    "ClassificationsResourceWithStreamingResponse",
    "AsyncClassificationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .qa_create_params import QaCreateParams as QaCreateParams
from .answer_extraction_response import AnswerExtractionResponse as AnswerExtractionResponse
